package eu.eudat.controllers;

import eu.eudat.exceptions.UnauthorisedException;
import eu.eudat.managers.InvitationsManager;
import eu.eudat.models.helpers.responses.ResponseItem;
import eu.eudat.models.invitation.Invitation;
import eu.eudat.models.security.Principal;
import eu.eudat.models.userinfo.UserInfoInvitationModel;
import eu.eudat.models.userinfo.UserInfoRequestItem;
import eu.eudat.services.ApiContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.UUID;

/**
 * Created by ikalyvas on 1/4/2018.
 */
@RequestMapping("invite/")
@RestController
@CrossOrigin
public class UserInvitationController extends BaseController{

    @Autowired
    public UserInvitationController(ApiContext apiContext) {
        super(apiContext);
    }

    @Transactional
    @RequestMapping(method = RequestMethod.POST, value = { "/users" }, consumes = "application/json", produces="application/json")
    public @ResponseBody ResponseItem<Invitation> users(@RequestBody Invitation invitation, Principal principal) {
        try {
            InvitationsManager.inviteUsers(this.getApiContext(),invitation,principal);
            return new ResponseItem<Invitation>().status(HttpStatus.OK).message("Users have beeen invited");
        } catch (Exception e) {
            e.printStackTrace();
            return new ResponseItem<Invitation>().message(e.getMessage());
        }
    }

    @Transactional
    @RequestMapping(method = RequestMethod.GET, value = { "/exchange/{invitationID}" }, produces="application/json")
    public @ResponseBody ResponseItem<UUID> exchange(@PathVariable UUID invitationID, Principal principal) {
        try {
            UUID dmpId = InvitationsManager.assignUserAcceptedInvitation(this.getApiContext(),invitationID,principal);
            return new ResponseItem<UUID>().status(HttpStatus.OK).payload(dmpId);
        }catch (UnauthorisedException e){
            e.printStackTrace();
            throw  e;
        }
        catch (Exception e) {
            e.printStackTrace();
            return new ResponseItem<UUID>().message(e.getMessage());
        }
    }

    @RequestMapping(method = RequestMethod.POST, value = { "/getUsers" }, consumes = "application/json", produces="application/json")
    public @ResponseBody ResponseItem<List<UserInfoInvitationModel>> getUsers(@RequestBody UserInfoRequestItem userInfoRequestItem) {
        try {
            List<UserInfoInvitationModel> users = InvitationsManager.getUsers(this.getApiContext(),userInfoRequestItem);
            return new ResponseItem<List<UserInfoInvitationModel>>().status(HttpStatus.OK).payload(users);
        } catch (Exception e) {
            e.printStackTrace();
            return new ResponseItem<List<UserInfoInvitationModel>>().status(HttpStatus.BAD_REQUEST).message(e.getMessage());
        }
    }
}
