package eu.eudat.dao.entities;

import java.util.List;
import java.util.UUID;

import javax.persistence.NoResultException;
import javax.persistence.TypedQuery;

import eu.eudat.dao.JpaDao;
import eu.eudat.dao.databaselayer.service.DatabaseService;
import eu.eudat.entities.DMP;
import eu.eudat.entities.Invitation;
import eu.eudat.entities.UserInfo;
import eu.eudat.models.criteria.ProjectCriteria;
import eu.eudat.models.criteria.UserInfoCriteria;
import eu.eudat.queryable.QueryableList;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component("userInfoDao")
public class UserInfoDaoImpl implements UserInfoDao {

    @Autowired
    private DatabaseService<UserInfo> databaseService;

    @Override
    public QueryableList<UserInfo> getWithCriteria(UserInfoCriteria criteria) {
        QueryableList<UserInfo> users = this.databaseService.getQueryable(UserInfo.class);
        if (criteria.getLike() != null)
            users.where((builder, root) -> builder.or(builder.like(root.get("name"), "%" + criteria.getLike() + "%"), builder.like(root.get("email"), "%" + criteria.getLike() + "%")));
        if(criteria.getEmail()!=null)
            users.where((builder, root) -> builder.equal(root.get("email"),criteria.getEmail()));
        return users;
    }

    @Override
    public UserInfo createOrUpdate(UserInfo item) {
        return this.databaseService.createOrUpdate(item,UserInfo.class);
    }

    @Override
    public UserInfo find(UUID id) {
        return this.databaseService.getQueryable(UserInfo.class).where((builder, root) -> builder.equal(root.get("id"),id)).toList().get(0);
    }

    @Override
    public Long count() {
        return null;
    }
}